;+
;FUNCTION file_retrieve(pathnames,local_data_dir=local_data_dir)
;Keywords:
;    local_data_dir:  local data directory;  'must end with a '/'
;    remote_data_dir:  remote data directory:  'must end with a  '/'
;
;$LastChangedBy: kenb-mac $
;$LastChangedDate: 2007-07-09 10:37:13 -0700 (Mon, 09 Jul 2007) $
;$LastChangedRevision: 1034 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/ssl_general/branches/QA/misc/file_retrieve.pro $
;-
function file_retrieve,pathnames,  structure_format=structure_format, $
    local_data_dir=local_data_dir,remote_data_dir=remote_data_dir, $
    no_download=no_download,no_update=no_update,verbose=verbose, $
    progress=progress,progobj=progobj

dprint,dlevel=4,'Start; $Id: file_retrieve.pro 1034 2007-07-09 17:37:13Z kenb-mac $'
if keyword_set(structure_format) then begin
   str= {   $
      retrieve_struct,       $
      init:0,                $
      local_data_dir:'',     $
      remote_data_dir:'',    $
      progress: 1    ,       $    ; Currently unused keyword   (progress is printed by default)
      progobj: obj_new(),    $    ; Experimental option for a progress bar widget.  (please ignore for now)
;      min_age_limit: 600L  , $    ; Really Dumb!!!!  Files younger than this age (in seconds) will are assumed current (avoids the need to check server)
      no_download:0   ,      $    ; Set to 1 to prevent downloads from remote server
      no_update:0     ,      $    ; Set to 1 to prevent updates from remote server
      downloadonly:0  ,      $    ; Set to 1 to only download files but not load into memory.
      verbose:2              $
   }
   return, str
endif

;if not keyword_set(local_data_dir) then   local_data_dir = './'
;if not keyword_set(remote_data_dir) then   remote_data_dir = ''
vb = keyword_set(verbose) ? verbose : 0
if n_elements(progress) eq 0 then progress=1

;if keyword_set(progress) then begin
;    progobj = obj_new('progressbar')
;endif


;fullnames = filepath(root_dir=local_data_dir, pathnames)
fullnames = local_data_dir + pathnames   ; trailing '/' is required on local_data_dir
n0 = n_elements(fullnames)

if keyword_set(remote_data_dir) and  not keyword_set(no_download) then begin
;    if vb ge 2 then  print,ptrace() ,string(format="('Retrieving ',i0,' files from ',a)",n0,remote_data_dir)
    If obj_valid(progobj) Then progobj -> update, 0.0, text = string(format="('Retrieving ',i0,' files from ',a)",n0,remote_data_dir) ;jmm, 15-may-2007
    for i = 0,n0-1 do begin
        fn = fullnames[i]
        pn = pathnames[i]
        if keyword_set(no_update) and file_test(fn,/regular) then continue
        file_http_copy,serverdir=remote_data_dir,localdir=local_data_dir,pathname=pn,error=error,verbose=verbose,progobj = progobj
;        if vb ge 1 and keyword_set(error) then print,ptrace(),error
    endfor
endif



; The following bit of code should find the highest version number if globbing is used.

for i=0,n_elements(fullnames)-1 do begin
   ff = file_search(fullnames[i],count=c)
   case c of
   0:    dprint,dlevel=2,verbose=vb,'No matching file: "'+fullnames[i]+'"'
   1:    begin
           fullnames[i] = ff[0]
           dprint,dlevel=5,verbose=vb,'Found: "'+fullnames[i]+'"'
         end
   else: begin
           fullnames[i] = ff[n_elements(ff)-1]   ; Cluge to Use highest version number?
           dprint,dlevel=2,verbose=vb,'Multiple matches found for: "'+fullnames[i]+'"'
         end
   endcase
endfor

return,fullnames
end

