;+
;NAME:
; thm_ui_choose_dtype
;PURPOSE:
; A widget for choosing data for the THEMIS data analysis GUI
;CALLING SEQUENCE:
; thm_ui_choose_dtype, instr, dtyp, station, astation, probe
;INPUT:
; instr = the instrument,  one of ['asi', 'ask', 'esa_pkt', $
;           'efi', 'fbk', 'fft', 'fgm', $
;           'fit', 'gmag', 'mom', 'scm', 'sst', 'state']
;OUTPUT:
; dtyp = the output datatype, a string of instr/datatype/datalevel,
;        e.g., 'state/spinras/l1'
; station = the ground station(s) if gmag data was chosen
; astation = the ground station(s) if asi data was chosen
; probe = the spacecraft if spacecraft data was chosen
;HISTORY:
; jmm, jimm@ssl.berkeley.edu 1-may-2007
; jmm, Changed the probe list to reflect probe assigments:
;      A (P5), B (P1), C (P2), D (P3) and E (P4)
;       14-jun-2007
; jmm, Now a no_block widget, jun-19-2007
;$LastChangedBy$
;$LastChangedDate$
;$LastChangedRevision$
;$URL$
;-
Pro thm_ui_choose_dtype_event, event
  Common dtypw, dtyp10, dtyp20, station0, astation0, probe0
  Common dtypw_info, stations, astations, probes, dlist1, dlist2, $
    dlist1_all, dlist2_all

  err_xxx = 0
  catch, err_xxx
  If(err_xxx Ne 0) Then Begin
    catch, /cancel
    help, /last_message, output = err_msg
    For j = 0, n_elements(err_msg)-1 Do print, err_msg[j]
    If(is_struct(state)) Then Begin
      cw = state.cw
      widget_control, event.top, set_uval = state, /no_copy
    Endif Else Begin
      widget_control, event.top, get_uval = state, /no_copy
      If(is_struct(state)) Then Begin
        cw = state.cw
        widget_control, event.top, set_uval = state, /no_copy
      Endif Else cw = -1
    Endelse
    If(widget_valid(cw)) Then Begin
      If(is_struct(wstate)) Then widget_control, cw, set_uval = wstate
      thm_ui_update_history, cw, [';*** FYI', ';'+err_msg]
      thm_ui_update_progress, cw, 'Error--See history'
    Endif
    thm_ui_error
    Return
  Endif

;  what happened?
  widget_control, event.id, get_uval = uval
  Case uval Of
    'EXIT': widget_control, event.top, /destroy
    'PRST':Begin
      widget_control, event.top, get_uval = state, /no_copy
      pindex = widget_info(state.prstlist, /list_select)
      all_chosen = where(pindex Eq 0, nall)
      If(state.instr Eq 'asi' Or state.instr Eq 'ask') Then Begin
        If(nall Gt 0) Then astation0 = astations[1:*] $
        Else astation0 = astations[pindex]
        widget_control, state.cw, get_uval = wstate, /no_copy
        If(ptr_valid(wstate.astation)) Then ptr_free, wstate.astation
        wstate.station = ptr_new(astation0)
        widget_control, state.cw, set_uval = wstate, /no_copy
        thm_ui_update_history, state.cw, $
          thm_ui_multichoice_history('asi_station = ', astation0)
      Endif Else If(state.instr Eq 'gmag') Then Begin
        If(nall Gt 0) Then station0 = stations[1:*] $
        Else station0 = stations[pindex]
        widget_control, state.cw, get_uval = wstate, /no_copy
        If(ptr_valid(wstate.station)) Then ptr_free, wstate.station
        wstate.station = ptr_new(station0)
        widget_control, state.cw, set_uval = wstate, /no_copy
        thm_ui_update_history, state.cw, $
          thm_ui_multichoice_history('gmag_station = ', station0)
      Endif Else Begin
        If(nall Gt 0) Then probe0 = probes[1:*] $
        Else probe0 = probes[pindex]
        widget_control, state.cw, get_uval = wstate, /no_copy
        If(ptr_valid(wstate.probe)) Then ptr_free, wstate.probe
        wstate.probe = ptr_new(probe0)
        widget_control, state.cw, set_uval = wstate, /no_copy
        thm_ui_update_history, state.cw, $
          thm_ui_multichoice_history('probe = ', probe0)
      Endelse
      widget_control, event.top, set_uval = state, /no_copy
    End
    'DTYP1':Begin
      widget_control, event.top, get_uval = state, /no_copy
      pindex = widget_info(state.dtyp1list, /list_select)
      all_chosen = where(pindex Eq 0, nall)
      If(dlist1[0] Ne 'None') Then Begin
        If(nall Gt 0) Then dtyp10 = dlist1[1:*] $
        Else dtyp10 = dlist1[pindex]
        If(state.instr Eq 'esa_pkt') Then Begin
          dtype = strmid(dtyp10, 0, 3)
        Endif Else dtype = dtyp10
        dtype = state.instr+'/'+dtype+'/l1'
        dtype = strcompress(strlowcase(dtype), /remove_all)
        widget_control, state.cw, get_uval = wstate, /no_copy
        If(ptr_valid(wstate.dtyp_1)) Then ptr_free, wstate.dtyp_1
        wstate.dtyp_1 = ptr_new(dtype)
        widget_control, state.cw, set_uval = wstate, /no_copy
      Endif
      widget_control, event.top, set_uval = state, /no_copy
    End
    'DTYP2':Begin
      widget_control, event.top, get_uval = state, /no_copy
      pindex = widget_info(state.dtyp2list, /list_select)
      all_chosen = where(pindex Eq 0, nall)
      If(dlist2[0] Ne 'None') Then Begin
        If(nall Gt 0) Then dtyp20 = dlist2[1:*] $
        Else dtyp20 = dlist2[pindex]
        dtype = dtyp20
        dtype = state.instr+'/'+dtype+'/l2'
        dtype = strcompress(strlowcase(dtype), /remove_all)
        widget_control, state.cw, get_uval = wstate, /no_copy
        If(ptr_valid(wstate.dtyp_1)) Then ptr_free, wstate.dtyp_1
        wstate.dtyp_1 = ptr_new(dtype)
        widget_control, state.cw, set_uval = wstate, /no_copy
      Endif
      widget_control, event.top, set_uval = state, /no_copy
    End
    'STRTEXT':Begin
      widget_control, event.top, get_uval = state, /no_copy
      widget_control, event.id, get_val = temp_string
      If(temp_string Eq '*') Then Begin
        dlist1 = dlist1_all
        dlist2 = dlist2_all
      Endif Else Begin
        ts = strcompress(temp_string, /remove_all)
        x1 = strmatch(dlist1, ts, /fold_case)
        tx1 = where(x1 Ne 0, ntx1)
        If(ntx1 Eq 0) Then dlist1 = dlist1_all $
        Else Begin
          y1 = where(dlist1[tx1] Eq '*')
          If(y1[0] Ne -1) Then dlist1 = dlist1[tx1] $
          Else dlist1 = ['*', dlist1[tx1]]
        Endelse
        ts = strcompress(temp_string, /remove_all)
        x2 = strmatch(dlist2, ts, /fold_case)
        tx2 = where(x2 Ne 0, ntx2)
        If(ntx2 Eq 0) Then dlist2 = dlist2_all $
        Else Begin
          y2 = where(dlist2[tx2] Eq '*')
          If(y2[0] Ne -1) Then dlist2 = dlist2[tx2] $
          Else dlist2 = ['*', dlist2[tx2]]
        Endelse
      Endelse
;reset the values of the datalists
      widget_control, state.dtyp1list, set_val = dlist1
      widget_control, state.dtyp2list, set_val = dlist2
      widget_control, event.top, set_uval = state, /no_copy
    End
  Endcase

Return
End
Pro thm_ui_choose_dtype, gui_id, instr_in0

  Common dtypw, dtyp10, dtyp20, station0, astation0, probe0
  Common dtypw_info, stations, astations, probes, dlist1, dlist2, $
    dlist1_all, dlist2_all

  arrx = 'Nothing Here'
  dtyp10 = '' & dtyp20 = '' & station0 = '' & astation0 = ''
  probe0 = '' & instr0 = ''

  instr_in = strlowcase(instr_in0)

;Get valid datatypes, probes, etc for different data types
  dlist = thm_ui_valid_dtype(ilist, instr_in = instr_in)
  If(instr_in Eq 'asi' Or instr_in Eq 'ask') Then Begin
    fl = 'All-Sky Ground Station'
    If(is_string(astations) Eq 0) Then Begin
      thm_load_asi, /valid_names, site = asi_stations
      astations = ['*', asi_stations]
      astations = strlowcase(strcompress(/remove_all, astations))
    Endif
    prst_val = astations
    dlist1_all = ['*', dlist]
    dlist2_all = 'None'
  Endif Else If(instr_in Eq 'gmag') Then Begin
    fl = 'GMAG Ground Station'
    If(is_string(stations) Eq 0) Then Begin
      thm_load_gmag, /valid_names, site = gmag_stations
      stations = ['*', gmag_stations]
      stations = strlowcase(strcompress(/remove_all, stations))
    Endif
    prst_val = stations
    dlist1_all = 'None'
    dlist2_all = ['*', dlist]
  Endif Else Begin
    fl = 'THEMIS Probe'
    probes = ['*', 'a', 'b', 'c', 'd', 'e']
    probes_ext = ['*', 'A (P5)',  'B (P1)',  'C (P2)',  'D (P3)', 'E (P4)']
    prst_val = probes_ext
    If(instr_in Eq 'esa_pkt') Then Begin
      ext = '--'+['peif', 'peir', 'peib', 'peef', 'peer', 'peeb']
      If(n_elements(dlist) Eq n_elements(ext)) Then dlist = dlist+ext
    Endif
    dlist1_all = ['*', dlist]
    dlist2_all = 'None'
  Endelse
  dlist1 = dlist1_all & dlist2 = dlist2_all
;Set up the widget
  thmds = widget_base(/col, title = 'THEMIS Science Software: '+$
                      strupcase(instr_in)+': DATA INPUT OPTIONS', $
                      tlb_frame_attr = 1, group_leader = gui_id)
;Three list widgets, probe/station, level1 quantities, level2
;quantities, One text widget for an input string
;The lists are on top?
  listsw = widget_base(thmds, /row, frame = 5, /align_center)
;The string choice input is on the bottom
  strsw = widget_base(thmds, /row, frame = 5, /align_center)
  prstw = widget_base(listsw, /col, frame = 5, /align_center)
  flabel = widget_label(prstw, value = fl)
  prstid = widget_base(prstw, /row, /align_center, /frame)
  prstlist = widget_list(prstid, value = prst_val, xsiz = 8, $
                         ysiz = 20, uval = 'PRST', /mult)
  dtyp1w = widget_base(listsw, /col, frame = 5, /align_center)
  flabel = widget_label(dtyp1w, value = 'Level 1 Data Quantity')
  dtyp1id = widget_base(dtyp1w, /row, /align_center, /frame)
  dtyp1list = widget_list(dtyp1id, value = dlist1, xsiz = 8, $
                         ysiz = 20, uval = 'DTYP1', /mult)
  dtyp2w = widget_base(listsw, /col, frame = 5, /align_center)
  flabel = widget_label(dtyp2w, value = 'Level 2 Data Quantity')
  dtyp2id = widget_base(dtyp2w, /row, /align_center, /frame)
  dtyp2list = widget_list(dtyp2id, value = dlist2, xsiz = 8, $
                         ysiz = 20, uval = 'DTYP2', /mult)
;input string button
  flabel = widget_label(strsw, value = 'String to Match')
  strtextw = widget_text(strsw, value = '*', $
                         xsiz = 12, $
                         ysiz = 1, uval = 'STRTEXT', $
                         /editable, /all_events)
;exit button
  exbut = widget_button(strsw, val = ' Accept and Close ', $
                        uval = 'EXIT', /align_center)
;state structure
  state = {cw:gui_id, dtypw_id:thmds, prstlist:prstlist, dtyp1list:dtyp1list, $
           dtyp2list:dtyp2list, instr:instr_in}
  widget_control, thmds, set_uval = state, /no_copy

;  realize
  widget_control, thmds, /realize
  xmanager, 'thm_ui_choose_dtype', thmds
End
