;+
;NAME:
; thm_ui_config
;PURPOSE:
; A widget that allows the user to set some of the fields in the
; !themis system variable: Also allows the user to set the themis
; configuration text file, and save it
;HISTORY:
; 17-may-2007, jmm, jimm@ssl.berkeley.edu
; 2-jul-2007, jmm, 'Add trailing slash to data directories, if necessary
;$LastChangedBy$
;$LastChangedDate$
;$LastChangedRevision$
;$URL$
;-
Pro thm_ui_config_event, event
  widget_control, event.id, get_uval = uval
  Case uval Of
    'LOCALDIR': Begin
      widget_control, event.id, get_val = temp_string
      temp_string = strtrim(temp_string, 2)
      ll = strmid(temp_string, strlen(temp_string)-1, 1)
      If(ll Ne '/' And ll Ne '\') Then temp_string = temp_string+'/'
      !themis.local_data_dir = temporary(temp_string)
    End
    'REMOTEDIR': Begin
      widget_control, event.id, get_val = temp_string
      temp_string = strtrim(temp_string, 2)
      ll = strmid(temp_string, strlen(temp_string)-1, 1)
      If(ll Ne '/' And ll Ne '\') Then temp_string = temp_string+'/'
      !themis.remote_data_dir = temporary(temp_string)
    End
    'NODOWNLOAD':Begin
      widget_control, event.id, get_val = temp_string
      If(event.type Le 2) Then !themis.no_download = fix(temp_string)
    End
    'NOUPDATE':Begin
      widget_control, event.id, get_val = temp_string
      If(event.type Le 2) Then !themis.no_update = fix(temp_string)
    End
    'DOWNLOADONLY':Begin
      widget_control, event.id, get_val = temp_string
      If(event.type Le 2) Then !themis.downloadonly = fix(temp_string)
    End
    'VERBOSE':Begin
      widget_control, event.id, get_val = temp_string
      If(event.type Le 2) Then !themis.verbose = fix(temp_string)
    End
    'RESET': Begin
      widget_control, event.top, get_uval = state, /no_copy
      !themis = state.thm_cfg_sav
      If(obj_valid(!themis.progobj)) Then Begin
        !themis.progobj -> update, 0.0, $
          text = 'Restoring original Configuration'
      Endif
      widget_control, state.localdir, set_val = !themis.local_data_dir
      widget_control, state.remotedir, set_val = !themis.remote_data_dir
      widget_control, state.nodownload, set_val = string(!themis.no_download)
      widget_control, state.noupdate, set_val = string(!themis.no_update)
      widget_control, state.downloadonly, set_val = string(!themis.downloadonly)
      widget_control, state.verbose, set_val = string(!themis.verbose)
      widget_control, event.top, set_uval = state, /no_copy
    End
   'RESETTODEFAULT': Begin
      dir =  thm_config_filedir(/app_query)
      If(dir[0] Ne '') Then Begin
;Is there a trailing slash? Not for linux or windows, not sure about
;Mac                                 
        ll =  strmid(dir, strlen(dir)-1, 1)
        If(ll Eq '/' Or ll Eq '\') Then filex =  dir+'thm_config.txt' $
        Else filex =  dir+'/'+'thm_config.txt'
;Does the file exist? If so, delete it
        If(file_search(filex) Ne '') Then file_delete, filex, /noexpand_path
      Endif
;No config file, so just run themis_init
      thm_init,  /reset
;Now you need to put the valuse that you have back into the widget
      widget_control,  event.top,  get_uval =  state,  /no_copy
      widget_control,  state.localdir,  set_val =  !themis.local_data_dir
      widget_control,  state.remotedir,  set_val =  !themis.remote_data_dir
      widget_control,  state.nodownload,  set_val =  string(!themis.no_download)
      widget_control,  state.noupdate,  set_val =  string(!themis.no_update)
      widget_control,  state.downloadonly,  set_val =  string(!themis.downloadonly)
      widget_control,  state.verbose,  set_val =  string(!themis.verbose)
      widget_control,  event.top,  set_uval =  state,  /no_copy
    End
    'SAVE': Begin
      thm_write_config
      If(obj_valid(!themis.progobj)) Then Begin
        filex = thm_config_filedir()+'/'+'thm_config.txt'
        !themis.progobj -> update, 0.0, $
          text = 'Saved New Configuration file: '+filex, /history_too
      Endif
    End
    'EXIT': Begin
      widget_control, event.top, /destroy
    End
  Endcase

Return
End
Pro thm_ui_config

;If !themis does not exist, set it
  thm_init
  thm_cfg_sav = !themis

;Build the widget
  master = widget_base(/col, $
                       title = 'THEMIS Science Software: Configuration', $
                       /tlb_frame_attr, /align_top)
;widget base for values to set
  vmaster = widget_base(master, /col, /align_left)
  vlabel = widget_label(vmaster, value = 'Configuration Settings')
;Widget base for save, reset and exit buttons
  bmaster = widget_base(master, /row, /align_left)
  ll = max(strlen([!themis.local_data_dir, !themis.remote_data_dir]))
;Now editable text widgets
  lbase = widget_base(vmaster, /row, /align_left)
  localdir = widget_text(lbase, /edit, /all_events, xsiz = ll, $
                         uval = 'LOCALDIR', val = !themis.local_data_dir)
  flabel = widget_label(lbase, value = 'Local data directory')
  rbase = widget_base(vmaster, /row, /align_left)
  remotedir = widget_text(rbase, /edit, /all_events, xsiz = ll, $
                          uval = 'REMOTEDIR', val = !themis.remote_data_dir)
  flabel = widget_label(rbase, value = 'Remote data directory')
  ndbase = widget_base(vmaster, /row, /align_left)
  nodownload = widget_text(ndbase, /edit, /all_events, xsiz = 10, $
                           uval = 'NODOWNLOAD', val = string(!themis.no_download))
  flabel = widget_label(ndbase, value = 'Set to 0 for auto downloads, 1 for no auto downloads')
  nubase = widget_base(vmaster, /row, /align_left)
  noupdate = widget_text(nubase, /edit, /all_events, xsiz = 10, $
                         uval = 'NOUPDATE', val = string(!themis.no_update))
  flabel = widget_label(nubase, value = 'Set to 0 to always update files, 1 to not update files')
  dwbase = widget_base(vmaster, /row, /align_left)
  downloadonly = widget_text(dwbase, /edit, /all_events, xsiz = 10, $
                             uval = 'DOWNLOADONLY', val = string(!themis.downloadonly))
  flabel = widget_label(dwbase, value = 'set to 0 to download and load data, set to 1 to only download')
  vbbase = widget_base(vmaster, /row, /align_left)
  verbose = widget_text(vbbase, /edit, /all_events, xsiz = 10, $
                        uval = 'VERBOSE', val = string(!themis.verbose))
  flabel = widget_label(vbbase, value = 'Verbose: from 0 to 10, higher value = more comments')

;buttons
  savebut = widget_button(bmaster, value = '     Save     ', uvalue = 'SAVE')
  resetbut = widget_button(bmaster, value = '     Reset     ', uvalue = 'RESET')
  reset_to_dbutton =  widget_button(bmaster,  value =  '  Reset to Default   ',  uvalue =  'RESETTODEFAULT')
  exitbut = widget_button(bmaster, value = '    Close     ', uvalue = 'EXIT')

  state = {thm_cfg_sav:thm_cfg_sav, $
          localdir:localdir, remotedir:remotedir, $
          nodownload:nodownload, noupdate:noupdate, $
          downloadonly:downloadonly, verbose:verbose}
  
  widget_control, master, set_uval = state, /no_copy
  widget_control, master, /realize
  xmanager, 'thm_ui_config', master, /no_block

End

 
