;+
;NAME:
; thm_ui_dproc
;PURPOSE:
; A widget interface for calling tplot from the themis_w widget,
; currently of limited usefulness, but expandable
;CALLING SEQUENCE:
; thm_ui_dproc, master_widget_id
;INPUT:
; master_widget_id = the id number of the widget that calls this
;OUTPUT:
; none, there are buttons to push for plotting, setting limits, not
; sure what else yet...
;HISTORY:
; 14-dec-2006, jmm, jimm@ssl.berkeley.edu
; 13-feb-2007, jmm, fixed problem with !p.background resetting in
; set_plot commands
; 12-mar-2007, jmm, Added wavelet button
; 2-apr-2007, jmm, Added pwrspc options, NEEDS TESTING.
; 5-jun-2007, jmm, rewritten
;
;$LastChangedBy: kenb-mac $
;$LastChangedDate: 2007-01-26 15:52:34 -0800 (Fri, 26 Jan 2007) $
;$LastChangedRevision: 241 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/trunk/idl/themis/common/thm_ui_dproc.pro $
;
;-
Pro thm_ui_dproc_event, event

  @tplot_com
  Common tlimits_saved, t0_sav, t1_sav, t0_init, t1_init

  err_xxx = 0
  catch, err_xxx
  If(err_xxx Ne 0) Then Begin
    catch, /cancel
    help, /last_message, output = err_msg
    For j = 0, n_elements(err_msg)-1 Do print, err_msg[j]
    If(is_struct(state)) Then Begin
      cw = state.cw
      widget_control, event.top, set_uval = state, /no_copy
    Endif Else Begin
      widget_control, event.top, get_uval = state, /no_copy
      If(is_struct(state)) Then Begin
        cw = state.cw
        widget_control, event.top, set_uval = state, /no_copy
      Endif Else cw = -1
    Endelse
    If(widget_valid(cw)) Then Begin
      If(is_struct(wstate)) Then widget_control, cw, set_uval = wstate
      thm_ui_update_history, cw, [';*** FYI', ';'+err_msg]
      thm_ui_update_progress, cw, 'Error--See history'
    Endif
    thm_ui_error
    Return
  Endif

;start here
  widget_control, event.id, get_uval = uval
  If(uval Eq 'EXIT') Then Begin
    widget_control, event.top, /destroy
  Endif Else If(uval Eq 'TLIMIT') Then Begin
    widget_control, event.top, get_uval = state, /no_copy
    cw = state.cw
    widget_control, event.top, set_uval = state, /no_copy
    thm_ui_update_progress, cw, 'Choosing Time Limits for Processing'
    thm_ui_set_tlimits, event.top
    widget_control, event.top, get_uval = state, /no_copy
    state.st_time = t0_sav & state.en_time = t1_sav
    widget_control, event.top, set_uval = state, /no_copy
  Endif Else If(uval Eq 'RESTORE') Then Begin
    widget_control, event.top, get_uval = state, /no_copy
    fff = dialog_pickfile(title = 'Choose Filename To Restore Data:', $
                          filter = '*.tplot')
    history_ext = 'tplot_restore, filenames = '+$
      ''''+fff+''''+', /get_tvars'
    thm_ui_update_progress, state.cw, 'Restoring Files: '+fff
    tplot_restore, filenames = fff, /get_tvars, restored_varnames = vn_new
    history_ext = [history_ext, $
                   thm_ui_multichoice_history('varnames = ', vn_new)]
    thm_ui_update_history, state.cw, history_ext
    thm_ui_update_progress, state.cw, 'Restored Files: '+fff
    thm_ui_update_data_all, state.cw, vn_new
    widget_control, event.top, set_uval = state, /no_copy
  Endif Else Begin
    widget_control, event.top, get_uval = state, /no_copy
    widget_control, state.cw, get_uval = wstate, /no_copy
    If(ptr_valid(wstate.active_vnames)) Then Begin
      tvn = *wstate.active_vnames
      widget_control, state.cw, set_uval = wstate, /no_copy
      vn_new = tvn              ;initialize new variable array
      Case uval Of
;subtracts the mean and creates new variables for plotting
        'SUBAVG':Begin
          vn_new = tvn+'-d'
;history of this is not easy...
          history_ext = ['new_vars = varnames+''-d'', $', $
                         'For j = 0, n_elements(varnames) -1 Do Begin &$', $
                         '  get_data, varnames[j], data=d &$', $
                         '  svalue = average(d.y, 1, /double)', $
                         '  d.y = d.y-(replicate(1, n_elements(d.x))#svalue) &$', $
                         '  tplot, /add, new_vars[j]', $
                         'varnames = new_vars']
          thm_ui_update_history, state.cw, history_ext
          thm_ui_update_progress, state.cw, 'Subtracting Average Values:'
          For j = 0, n_elements(tvn)-1 Do Begin
            get_data, tvn[j], data = d
            svalue = average(d.y, 1, /double)
            szsv = size(svalue)
            If(szsv[0] Le 1) Then Begin
              d.y = d.y-(replicate(1, n_elements(d.x))#svalue)
              store_data, vn_new[j], data = temporary(d)
            Endif Else If(szsv[0] Eq 2) Then Begin
              tdata = rebin(svalue, szsv[1], szsv[2], n_elements(d.x))
              tdata = transpose(tdata, [2, 0, 1])
              d.y = d.y-temporary(tdata)
              store_data, vn_new[j], data = temporary(d)
            Endif Else Begin
              thm_ui_update_progress, state.cw, 'Operation not supported for data arrays of more than 2d'
            Endelse
          Endfor
          thm_ui_update_progress, state.cw, 'Done Subtracting Average Values:'
        End
;subtracts the mean and creates new variables for plotting
        'SUBMED':Begin
          vn_new = tvn+'-m'
;history of this is not easy...
          history_ext = ['new_vars = varnames+''-d'', $', $
                         'For j = 0, n_elements(varnames) -1 Do Begin &$', $
                         '  get_data, varnames[j], data=d &$', $
                         '  svalue = average(d.y, 1, /double,/ret_median)', $
                         '  d.y = d.y-(replicate(1, n_elements(d.x))#svalue) &$', $
                         '  tplot, /add, new_vars[j]', $
                         'varnames = new_vars']
          thm_ui_update_history, state.cw, history_ext
          thm_ui_update_progress, state.cw, 'Subtracting Median Values:'
          For j = 0, n_elements(tvn)-1 Do Begin
            get_data, tvn[j], data = d
            svalue = average(d.y, 1, /double, /ret_median)
            szsv = size(svalue)
            If(szsv[0] Le 1) Then Begin
              d.y = d.y-(replicate(1, n_elements(d.x))#svalue)
              store_data, vn_new[j], data = temporary(d)
            Endif Else If(szsv[0] Eq 2) Then Begin
              tdata = rebin(svalue, szsv[1], szsv[2], n_elements(d.x))
              tdata = transpose(tdata, [2, 0, 1])
              d.y = d.y-temporary(tdata)
              store_data, vn_new[j], data = temporary(d)
            Endif Else Begin
              thm_ui_update_progress, state.cw, 'Operation not supported for data arrays of more than 2d'
            Endelse
          Endfor
          thm_ui_update_progress, state.cw, 'Done Subtracting Median Values:'
        End
        'SMOOTH':Begin
;Get the smoothing parameter
          smooth_res = thm_ui_par('Smoothing Resolution', '11')
          smooth_res = smooth_res[0]
          smr = strcompress(smooth_res, /remove_all)
          smooth_res = long(smooth_res)
          If(smooth_res Gt 0) Then Begin
            vn_new = tvn+'_sm_'+smr
            history_ext = ['width = '+smr, $
                           'new_vars = varnames+''_sm_'+smr+'''', $
                           'For j = 0, n_elements(new_vars)-1 Do', $
                           '  tsmooth2, varnames[j], width, newname = new_vars[j]', $
                           'varnames = new_vars']
            thm_ui_update_history, state.cw, history_ext
            thm_ui_update_progress, state.cw, 'Smoothing Data:'
            For j = 0, n_elements(vn_new)-1 Do $
              tsmooth2, tvn[j], smooth_res, newname = vn_new[j]
            thm_ui_update_progress, state.cw, 'Done Smoothing Data:'
          Endif Else thm_ui_update_progress, state.cw, 'Invalid smoothing Parameter'
        End
        'AVG':Begin
;Get the time_resolution
          time_res = thm_ui_par('Time Resolution (sec)', '60')
          time_res = time_res[0]
          smr = strcompress(time_res, /remove_all)
          time_res = float(time_res)
           If(time_res Gt 0) Then Begin
            vn_new = tvn+'_av_'+smr
            history_ext = ['time_res = '+smr, $
                           'new_vars = varnames+''_av_'+smr+'''', $
                           'For j = 0, n_elements(new_vars)-1 Do $', $
                           ' avg_data, varnames[j], time_res, newname = new_vars[j]', $
                           'varnames = new_vars']
            thm_ui_update_history, state.cw, history_ext
            thm_ui_update_progress, state.cw, 'Averaging Data:'
            For j = 0, n_elements(vn_new)-1 Do $
              avg_data, tvn[j], time_res, newname = vn_new[j]
            thm_ui_update_progress, state.cw, 'Done Averaging Data:'
         Endif Else thm_ui_update_progress, state.cw, 'Invalid Time Resolution'
         End
        'DERIV':Begin
          vn_new = 'd_'+tvn
          history_ext = ['new_vars = ''d_''+varnames', $
                         'For j = 0, n_elements(new_vars)-1 Do $', $
                         ' deriv_data, varnames[j], newname = new_vars[j]', $
                         'varnames = new_vars']
          thm_ui_update_history, state.cw, history_ext
          thm_ui_update_progress, state.cw, 'Taking Time Derivative:'
          For j = 0, n_elements(vn_new)-1 Do $
            deriv_data, tvn[j], newname = vn_new[j]
          thm_ui_update_progress, state.cw, 'Done Taking Time Derivative:'
        End
        'SPIKE':Begin
          vn_new = tvn+'_dspk'
          history_ext = ['new_vars = varnames+''_dspk''', $
                         'For j = 0, n_elements(new_vars)-1 Do $', $
                         ' clean_spikes, varnames[j], newname = new_vars[j]', $
                         'varnames = new_vars']
          thm_ui_update_history, state.cw, history_ext
          thm_ui_update_progress, state.cw, 'Despiking Data:'
          For j = 0, n_elements(vn_new)-1 Do $
            clean_spikes, tvn[j], new_name = vn_new[j]
          thm_ui_update_progress, state.cw, 'Done Despiking Data:'
        End
        'CLIP':Begin
          xxx = thm_ui_par('Min and Max for clipping', ['-20.0', '20.0'])
          amin = min(float(xxx)) & amax = max(float(xxx))
          vn_new = tvn+'_clip'
          thm_ui_update_progress, state.cw, 'Clipping Data:'
          history_ext = ['new_vars = varnames+''_clip''', $
                         'tclip, varnames, '+xxx[0]+', '+xxx[1]+$
                         ', newname = new_vars', 'varnames = new_vars']
          thm_ui_update_history, state.cw, history_ext
          tclip, tvn, amin, amax, newname = vn_new
          thm_ui_update_progress, state.cw, 'Done Clipping Data:'
        End
        'DEFLAG':Begin
          xxx = thm_ui_par('deflag method: LINEAR (Interpolate) or REPEAT (Last Value', 'repeat')
          method = strtrim(strlowcase(xxx[0]), 2)
          If(method Eq 'linear' Or method Eq 'repeat') Then Begin
            vn_new = tvn+'_deflag'
            history_ext = ['new_vars = varnames+''_deflag''', $
                           'tdeflag, varnames, '+method+', newname = new_vars', $
                           'varnames = new_vars']
            thm_ui_update_history, state.cw, history_ext
            thm_ui_update_progress, state.cw, 'Deflagging Data:'
            tdeflag, tvn, method, newname = vn_new
            thm_ui_update_progress, state.cw, 'Done Deflagging Data:'
          Endif Else thm_ui_update_progress, state.cw, 'Invalid Method Input'
        End
;degap button
        'DEGAP':Begin
          xxx = thm_ui_par('Time Interval, Margin, max gapsize for degapping', $
                           ['1.0', '0.25', '10000'])
          dtx = double(xxx[0])
          mrx = double(xxx[1])
          maxgapx = long(xxx[2])
          If(dtx Gt 0 And mrx Ge 0 And maxgapx Gt 0) Then Begin
            xxx = strcompress(xxx, /remove_all)
            vn_new = tvn+'_degap'
            history_ext = ['new_vars = varnames+''_degap''', $
                           'tdegap, varnames, newname = new_vars, dt = '+$
                           xxx[0]+', margin = '+xxx[1]+', maxgap = '+xxx[2], $
                           'varnames = new_vars']
            thm_ui_update_history, state.cw, history_ext
            thm_ui_update_progress, state.cw, 'Degapping Data:'
            tdegap, tvn, dt = dtx, margin = mrx, maxgap = maxgapx, $
              newname = vn_new
            thm_ui_update_progress, state.cw, 'Done Degapping Data:'
          Endif Else thm_ui_update_progress, state.cw, 'Invalid Parameters input:'
        End
;wavelet transform?
        'WV':Begin
          history_ext = ['thm_ui_wavelet, varnames, new_vars, [start_time, end_time]',  'varnames = new_vars']
          thm_ui_update_history, state.cw, history_ext
          thm_ui_update_progress, state.cw, 'Wavelet Transform:'
          thm_ui_wavelet, tvn, vn_new, [state.st_time, state.en_time]
          thm_ui_update_progress, state.cw, 'Finished Wavelet Transform:'
        End
;Dynamic Power Spectrum?
        'DPWRSPC':Begin
          history_ext = ['thm_ui_pwrspec, varnames, new_vars, [start_time, end_time],/dynamic',  'varnames = new_vars']
          thm_ui_update_history, state.cw, history_ext
          thm_ui_update_progress, state.cw, 'Dynamic Power Spectrum:'
          thm_ui_pwrspc, tvn, vn_new, [state.st_time, state.en_time], /dynamic
          thm_ui_update_progress, state.cw, 'Finished Dynamic Power Spectrum:'
        End
        'DELETE':Begin
          history_ext = 'store_data, varnames, /delete'
          thm_ui_update_history, state.cw, history_ext
          thm_ui_update_progress, state.cw, 'Deleting Data'
          store_data, tvn, /delete
          thm_ui_update_progress, state.cw, 'Done Deleting Data'
          widget_control, state.cw, get_uval = wstate, /no_copy
          If(ptr_valid(wstate.active_vnames)) Then $
            ptr_free, wstate.active_vnames
          widget_control, state.cw, set_uval = wstate, /no_copy
        End
        'SAVE': Begin
          xt = time_string(systime(/sec))
          ttt = strmid(xt, 0, 4)+strmid(xt, 5, 2)+strmid(xt, 8, 2)+$
            '_'+strmid(xt,11,2)+strmid(xt,14,2)+strmid(xt,17,2)
          fff = 'themis_saved_'+ttt
          fff = dialog_pickfile(title = 'Filename For Saved Data:', $
                                filter = '*.tplot', file = fff)
          history_ext = 'tplot_save, varnames, filename = '+''''+fff+''''
          thm_ui_update_history, state.cw, history_ext
          thm_ui_update_progress, state.cw, 'Saving Data'
          tplot_save, tvn, filename = fff
          thm_ui_update_progress, state.cw, 'Saved Data in File: '+fff
        End
        'SAVEASCII': Begin
          history_ext = 'tplot_ascii, varnames, trange = [' + $
            ''''+time_string(state.st_time)+''''+', '+ $
            ''''+time_string(state.en_time)+''''+']'
          thm_ui_update_history, state.cw, history_ext
          thm_ui_update_progress, state.cw, 'Saving Data in Ascii files'
          tplot_ascii, tvn, trange = [state.st_time, state.en_time]
          thm_ui_update_progress, state.cw, 'Finished Saving Data'
        End
        'RENAME': Begin
          ntvn = n_elements(tvn)
          For j = 0, ntvn-1 Do Begin
            pj = thm_ui_par('New Name for variable:'+tvn[j], tvn[j])
            history_ext = ['store_data,'+''''+tvn[j]+''''+$
                           ', newname ='+''''+pj+'''', $
                           'store_data,'+''''+tvn[j]+''''+', /delete']
            thm_ui_update_history, state.cw, history_ext
            thm_ui_update_progress, state.cw, 'Renaming Data'
            store_data, tvn[j], newname = pj
            store_data, tvn[j], /delete
            thm_ui_update_progress, state.cw, 'Renamed: '+tvn[j]+' to '+pj
            vn_new[j] = pj
          Endfor
        End
      Endcase
      thm_ui_update_data_all, state.cw, vn_new
    Endif Else Begin
      widget_control, state.cw, set_uval = wstate, /no_copy
      thm_ui_update_progress, state.cw, 'No Active Dataset, Nothing happened'
    Endelse
    widget_control, event.top, set_uval = state, /no_copy
  Endelse

  Return
End

Pro thm_ui_dproc, gui_id


;build master widget
  master = widget_base(/row, title = 'THEMIS Science Software: Data Processing ', $
                             scr_xsize = 330, /tlb_frame_attr, group_leader = gui_id)
;button widgets
  buttons = widget_base(master, /col, /align_left, frame = 5)
  buttons1 = widget_base(master, /col, /align_left, frame = 5)

;average subtraction button
  subavgbut = widget_button(buttons, val = ' Subtract Average', $
                            uval = 'SUBAVG', scr_xsize = 120)
;median subtraction button
  submedbut = widget_button(buttons, val = ' Subtract Median', $
                            uval = 'SUBMED', scr_xsize = 120)
;tsmooth2 button
  smoothbut = widget_button(buttons, val = ' Smooth Data', uval = 'SMOOTH', $
                            scr_xsize = 120)
;average button
  avgbut = widget_button(buttons, val = ' Time Average', uval = 'AVG', $
                         scr_xsize = 120)
;clip button
  clipbut = widget_button(buttons, val = ' Clip', uval = 'CLIP', $
                         scr_xsize = 120)
;dflag button
  deflagbut = widget_button(buttons, val = ' Deflag', uval = 'DEFLAG', $
                            scr_xsize = 120)
;degap button
  degapbut = widget_button(buttons, val = ' Degap', uval = 'DEGAP', $
                         scr_xsize = 120)
;dspike button
  spikebut = widget_button(buttons, val = ' Clean Spikes', uval = 'SPIKE', $
                         scr_xsize = 120)
;deriv button
  derivbut = widget_button(buttons, val = ' Time Derivative', uval = 'DERIV', $
                         scr_xsize = 120)
;wavelet button
  wavebut = widget_button(buttons, val = ' Wavelet Transform', uval = 'WV', $
                         scr_xsize = 120)
;power_spectrum button
;  pwrbut = widget_button(buttons, val = ' Power Spectrum', uval = 'PWRSPC', $
;                         scr_xsize = 120)
;dynamic power_spectrum button
  dpwrbut = widget_button(buttons, val = ' Dpwrspec', $
                          uval = 'DPWRSPC', scr_xsize = 120)


;tlimit button
  tlimitbut = widget_button(buttons1, val = ' Time limits ', $
                            uval = 'TLIMIT', /align_center)

;rename button
  renamebut = widget_button(buttons1, val = ' Rename ', uval = 'RENAME', $
                          scr_xsize = 120)
;save button
  savebut = widget_button(buttons1, val = ' Save ', uval = 'SAVE', $
                          scr_xsize = 120)
;restore button
  savebut = widget_button(buttons1, val = ' Restore ', uval = 'RESTORE', $
                          scr_xsize = 120)
;save ascii button
  savebut = widget_button(buttons1, val = ' Save Ascii ', uval = 'SAVEASCII', $
                          scr_xsize = 120)
;delete button
  deletebut = widget_button(buttons1, val = ' Delete ', uval = 'DELETE', $
                            scr_xsize = 120)
;exit button
  exitbut = widget_button(buttons1, val = ' Close ', uval = 'EXIT', $
                          scr_xsize = 120)

  cw = gui_id 
  widget_control, cw, get_uval = wstate, /no_copy
  wstate.proc_id = master
  t0 = wstate.st_time & t1 = wstate.en_time
  widget_control, cw, set_uval = wstate, /no_copy

; State structure
  state = {master:master, cw:cw, st_time:t0, en_time:t1}

  widget_control, master, set_uval = state, /no_copy
  widget_control, master, /realize
  xmanager, 'thm_ui_dproc', master, /no_block

  Return
End
