;+
;	Batch File: THM_CRIB_FBK
;
;	Purpose:  Demonstrate the loading, calibration, and plotting
;		of THEMIS FBK (Filter Bank) spectral data.
;
;	Calling Sequence:
;	.run thm_crib_fbk, or using cut-and-paste.
;
;	Arguements:
;   None.
;
;	Notes:
;	None.
;
; $LastChangedBy: pcruce $
; $LastChangedDate: 2007-06-22 15:42:12 -0700 (Fri, 22 Jun 2007) $
; $LastChangedRevision: 857 $
; $URL $
;-

; FBK FilterBank data example.

;start with a clean slate
del_data, '*'

print, 'at each stop point type .c to continue with the crib'

stop

;set a few TPLOT options. 
tplot_title = 'THEMIS FBK Spectra Examples'
tplot_options, 'title', tplot_title
tplot_options, 'xmargin', [ 15, 10]
tplot_options, 'ymargin', [ 5, 5]

; set the timespan and load the FBK data (FB1, FB2, FBH).
timespan, '2007-03-23', 1.0, /day

;this line loads the support data and the calibrated fbk data
thm_load_fbk, probe = 'a', /get_support_data

tplot_names

print, 'I Just loaded the calibrated and support data for fbk'
print, 'The list above is the set of tplot_variables'

stop

;this line load the raw data
;the suffix argument prevents the raw data from overwriting the
;original data
thm_load_fbk, probe = 'a', type = 'raw', suffix = '_raw'

tplot_names

print, 'Now I loaded the raw data for fbk'
print, 'Notice the list of tplot_variables now includes data with the suffix _raw'

stop

; set the color table.
loadct2, 39

; set some plotting options for the FBK data using OPTIONS.
;edited out but here if you want to use em
;options, 'th?_fb1_raw', 'colors', [ 1, 2, 3, 4, 5, 6]
;options, 'th?_fb1_raw', 'labels', [ '1', '2', '3', '4', '5', '6']
;options, 'th?_fb1_raw', 'labflag', 1

;options, 'th?_fb2_raw', 'colors', [ 1, 2, 3, 4, 5, 6]
;options, 'th?_fb2_raw', 'labels', [ '1', '2', '3', '4', '5', '6']
;options, 'th?_fb2_raw', 'labflag', 1

;options, 'th?_fbh_raw', 'colors', [ 1, 2]
;options, 'th?_fbh_raw', 'labels', [ 'peak', 'avg']
;options, 'th?_fbh_raw', 'labflag', 1
;options, 'th?_fbh_raw', 'ylog', 1
;options, 'th?_fbh_raw', 'yrange', [ 1.0e-5, 1.0]
;options, 'th?_fbh_raw', 'ystyle', 1

;options, 'th?_fb1', 'colors', [ 1, 2, 3, 4, 5, 6]
;options, 'th?_fb1', 'labels', [ '1', '2', '3', '4', '5', '6']
;options, 'th?_fb1', 'labflag', 1

;options, 'th?_fb2', 'colors', [ 1, 2, 3, 4, 5, 6]
;options, 'th?_fb2', 'labels', [ '1', '2', '3', '4', '5', '6']
;options, 'th?_fb2', 'labflag', 1

;options, 'th?_fbh', 'colors', [ 1, 2]
;options, 'th?_fbh', 'labels', [ 'peak', 'avg']
;options, 'th?_fbh', 'labflag', 1
;options, 'th?_fbh', 'ylog', 1
;options, 'th?_fbh', 'yrange', [ 1.0e-5, 1.0]
;options, 'th?_fbh', 'ystyle', 1

; TPLOT the RAW FBK data.
tplot, [ 'tha_fb1_raw']

print, 'I just plotted the raw filterbank 1 data from themis alpha'

stop

tlimit, '2007-03-23/19:54:00', '2007-03-23/20:30:00'

print, 'Now I adjusted the time limit to zoom in'

stop

tlimit, '2007-03-23/00:00:00', '2007-03-23/00:00:00'

print, 'Now I zoom back out'

stop

; plot the RAW and CALIBRATED FBK data for comparison.
tplot, [ 'tha_fb?_raw', 'tha_fb?']

print, 'Now I plotted the raw and calibrated data, notice the use of ? wildcards'

stop

options, 'th?_fb?_src', 'panel_size', 0.5
options, 'th?_fb?_src', 'psym', 4
options, 'th?_fb?_src', 'yrange', [ -1., 16.]
options, 'th?_fb?_src', 'ystyle', 1

tplot, [ 'tha_fb1_raw', 'tha_fb1', 'tha_fb1_src', 'tha_fb2_raw', 'tha_fb2', 'tha_fb2_src', 'tha_fbh_raw', 'tha_fbh']

print, 'Now we plot all the data'

stop

del_data, '*'

print, 'I deleted all the data'

stop

thm_load_fbk, probe = ['a', 'b'], datatype = ['fb2']

print, 'and now I loaded all the filterbank 1 data from probes alpha and beta'

tplot_names

stop

tplot, '*'

print, 'and plotted it using the * wildcard'

stop

thm_load_fbk, /valid_names

print, 'Use the /valid_names option to get a list of valid parameters for the fbk data type'

print, 'Now we are done'

end
