;+
;	Batch File: THM_CRIB_FIT
;
;	Purpose:  Demonstrate the loading, calibration, and plotting
;		of THEMIS FIT (On-Board E- and B-Field SpinFit) data.
;
;	Calling Sequence:
;	.run thm_crib_fit, or using cut-and-paste.
;
;	Arguements:
;   None.
;
;	Notes:
;	None.
;
;Written by John Bonnell
; $LastChangedBy: kramer $
; $LastChangedDate: 2007-06-22 10:20:30 -0700 (Fri, 22 Jun 2007) $
; $LastChangedRevision: 851 $
; $URL $
;-

; FIT On-Board SpinFit data example.

; set a few TPLOT options.
tplot_title = 'THEMIS FIT On-Board Spin Fit Examples'
tplot_options, 'title', tplot_title
tplot_options, 'xmargin', [ 15, 10]
tplot_options, 'ymargin', [ 5, 5]

; set the timespan and load the FIT data.
timespan, '2006-08-08', 1.0, /day

thm_load_state,probe='c', /get_support_data

thm_load_fit, level=1,datatype=['fit_efit', 'efs', 'fit_bfit', 'fgs'],/verbose,/get_support_data

; set the color table.
loadct2, 39

; the FIT data, as loaded by THM_LOAD_FIT isn't very usable or viewable,
; so run THM_CAL_FIT to break out the E and B fits with useful plotting options.

; plot the calibrated FIT data.
tplot, [ 'tha_fit_efit', 'tha_efs', 'tha_fit_bfit', 'tha_fgs']

stop

thm_load_state, probe='a', /get_support_data

thm_cotrans, 'tha_fgb', 'tha_fgs_gse', out_coord='gsm'

tplot, [ 'tha_fit_efit', 'tha_fit_efs', 'tha_fit_bfit', 'tha_fit_fgs', $
	'tha_fit_b_gse', 'tha_fit_b_gsm']

end
