;+
; **********************************************************************
; thm_crib_scm.pro : IDL batch file to get and calibrate SCM data
;  with thm_cal_scm
; K. Bromund
; O. Le Contel & P. Robert, CETP
; **********************************************************************
; $LastChangedBy: kenb-mac $
; $LastChangedDate: 2007-06-27 09:42:59 -0700 (Wed, 27 Jun 2007) $
; $LastChangedRevision: 894 $
; $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/branches/QA/idl/themis/examples/thm_crib_scm.pro $
;-

;;    ============================
;; 1) Select date an time interval
;;    ============================

  date = '2007-03-14/00:00:00'
  timespan,date,1,/day

;;    =====================
;; 2) Select probe and mode
;;    =====================

;; Select SATNAME (a,b,c,d or e)
  satname = 'd'

;; Select MODE (scf, scp, scw)
  mode = 'scp'

;;    ==================================
;; 3) get auxiliary data from STAT file
;;    ==================================

thm_load_state, probe=satname, /get_support_data

;;    ==================================
;; 4) get SCM data and SCM header file
;;    ==================================

;; If you want to set up a limited timespan for calibration,
;; you set up a trange array like this:

;; To impose by hand t1 and t2 :
;  starting_date =strmid(date,0,10)
;  starting_time='05:45:06.0'
;  ending_time  ='05:47:09.0'
;  starting_time='14:48:13.0'
;  ending_time  ='14:49:09.0'
 
;  trange = [starting_date+'/'+starting_time, $
;            starting_date+'/'+ending_time]
;;  don't forget to uncomment the trange keyword to thm_load_scm/thm_cal_scm!
 
;;    ===================================
;; 4a) easy method, including calibration
;;    ===================================

;; load and calibrate data of specified type:
thm_load_scm,probe=satname, datatype=mode, level=1 , trange=trange

thscs_mode = 'th'+satname+'_'+mode

tplot, thscs_mode, trange=trange

stop

;;    =============================================
;; 4b) method for customized and diagnostic calibration
;;    =============================================

;; get data over a whole day

thm_load_scm,probe=satname, level=1,  type='raw', /get_support

;; get the raw data in volts (step 1 output) in initial version, this
;; output was always created.
thm_cal_scm, probe=satname, datatype=mode, out_suffix = '_volt', step = 1

;; default values of parameters are shown.
;; the '*' appended to mode will get diagnostic parameters 
;; (_iano, _dc, _misalign) as well as calibrated output.  Note out_suffix
;; is necessary to get what was former default behavior (_cal on output)
;; To run with different parameters, uncomment and change as
;; you like.
;; Note: the /edge_zero is not a default option, so the output will differ 
;; slightly from step 4a) above.
thm_cal_scm, probe=satname, datatype=mode+'*', out_suffix = '_cal', $
             trange=trange, $
;             nk  = 512, $
;             mk = 4, $
;             Despin=1, $
;             N_spinfit = 2, $
;             Fdet = 2., $
;             Fcut = 0.1, $
;             Fmin = 0., $
;             Fmax = 0., $
;             step = 4, $
             /edge_zero

;;    =========================================
;; 5) Plot calibrated data
;;    =========================================

; ytitles are set as follows by thm_cal_scm:
;  options, thscs_mode+'_cal', 'YTITLE', $
;           'th'+satname+' '+mode+' '+str_Fsamp+' (nT)'

;; label the plot with calibration parameter string, which
;; can be retrieved from the metadata:

  get_data, thscs_mode+'_cal', dl = dl

  tplot_options, 'title', 'SCM calibrated data'
  tplot_options, 'subtitle', dl.data_att.str_cal_param
  tplot, thscs_mode+'_cal', trange=trange

  
  PRINT, 'Ready for other processing'
  PRINT, 'End of thm_crib_scm.pro batch file'
  PRINT, '-----------------------------------------'

;; ********************************************
;; end of thm_crib_scm.pro
;; ********************************************

end
