; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

  PRO scm_create_rff, Dir_result, rffname, time_scm,val_scm,satname, $
                      mode, time_state, spinpha,spinper, $
                      spinrasc,spindecl, time_head, TMrate

; ----------------------------------------------------------------------
; take scm data, state_file data and scm_feader data to create
; a RFF file including all we need to calibrate SCM data.
;
; P. Robert, CETP, January 2007
;
; ----------------------------------------------------------------------

  PRINT
  PRINT, '----------------------------------------------------------'
  PRINT, 'scm_create_rff: create a RFF file containing scm waveform '
  PRINT, '----------------------------------------------------------'
  PRINT

; Metadata definitions

  IF (satname EQ 'a') THEN satnum=1
  IF (satname EQ 'b') THEN satnum=2
  IF (satname EQ 'c') THEN satnum=3
  IF (satname EQ 'd') THEN satnum=4
  IF (satname EQ 'e') THEN satnum=5

  subtitle='THEMIS / SCM-wf / '+mode +' / th' +satname $
                           +' (#' +STRING(satnum,format='(I1)') +')'
  forout='(a29,1x,f5.0,1x,f10.6,3(1x,e14.6))'  ; a20 +nbdec +1 for 'Z'
  count=N_ELEMENTS(time_scm)

; ISO date time ; Themis time is now from 1970/01/01

  dursec=time_scm(count-1L) - time_scm(0)

  PRINT, 'RFF_name: ',rffname
  PRINT, 'Time conversion in ISODATIM format:'

  PRINT, '   starting at ', SYSTIME(0)
  PRINT, "   It's will take about 0.6 sec/mn, for 128 Hz on a classic PC..."
  PRINT, '   Duration is ',dursec, ' sec. or ',dursec/60.,' mn'
  PRINT, '   Present sample frequency is ', TMrate(0)
  PRINT, '   Please wait a few secondes ...'

  FLUSH, 0, -1, -2

  isodatim=tu_julsec_to_isodatea(time_scm,8)

  PRINT, '   ending   at ', SYSTIME(0)

; FOR i=0L,count-1L DO PRINT, time_scm(i),time_scm(i)+978307200.D, $
;              isodatim(i), $ FORMAT='(f15.10,1x,f29.13,1x,a)'

; interpollation of sample rate and spin period, aligned on scm time

  PRINT
  PRINT, 'Sample rate interpolation...'
  PRINT, '   starting at ', SYSTIME(0)
  FLUSH, 0, -1, -2
  scm_rate_interpol, time_head,TMrate,time_scm,TMrate_int
  PRINT, '   ending   at ', SYSTIME(0)

  PRINT
  PRINT, 'Spin interpollation...'
  PRINT, '   starting at ', SYSTIME(0)
  PRINT, '   Please wait a few secondes ...'
  FLUSH, 0, -1, -2
  thm_spin_phase, time_scm, SP_phase, probe=satname
;  scm_spin_interpol, time_state,spinpha,spinper,time_scm,SP_phase
  PRINT, '   ending   at ', SYSTIME(0)
  FLUSH, 0, -1, -2

; ----------------------------------------------------------------------

; open rff file & fill file with header, constant data and indexed data

  GET_LUN, unit

  PRINT, ' '
  PRINT, FORMAT='(3a,i4,a)', $
         'Creating ',rffname,' on unit',unit,', Please wait...'
  OPENW, unit, dir_result +rffname

  PRINT, '     starting at ', SYSTIME(0)
  PRINT, '     Fill file with RFF header...'

  scm_wrff_header, unit, rffname, satname, satnum, $
                             mode, forout, subtitle, count

  PRINT, '     Fill file with constant data...'

  scm_wrff_condat, unit, time_state, spinper, spinrasc, spindecl

  PRINT, '     Fill file with Time Vector data ...'

  FOR n=0L, count-1L DO BEGIN
  PRINTF, unit, FORMAT=forout, isodatim(n),TMrate_int(n),SP_phase(n), $
                               val_scm(n,0),val_scm(n,1),val_scm(n,2)
  ENDFOR



; write RFF EOF on file

  PRINT, '     Write RFF EOF on file ...'
  scm_wrff_eof, unit

  FREE_LUN, unit
  PRINT, '     ending   at ', SYSTIME(0)

; ----------------------------------------------------------------------

  PRINT
  PRINT, 'It''s done, RFF file created. Nb of vectors written=', count $
       , FORMAT='(a,i9)'
  PRINT, rffname,' is ready'
  PRINT, '------------------------------------------------------------'
  PRINT

  END

; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
