; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

  PRO scm_define_calparam, Dir_results, RFF_name, Cal_path,  $
                 Julsec1,Julsec2, nbp,despin,fdet,fcut,fmin,fmax,step

;-----------------------------------------------------------------------
; Create cowave.in from arguments
;
; Patrick Robert, CETP, February 2007
;
;-----------------------------------------------------------------------
                             
; Dir_Results: Path where cowave.in will be created
; RFF_name   : name of RFF file to read
; Cal_path   : path of cal directory
; Julsec1    : 1970 Jul. sec. of starting time
; Julsec2    : 1970 Jul. sec. of ending time
; nbp        : number of point of the cal window
; despin     : 1= classic despin, 0= nothing to do
; fdet       : detrend frequency; if set to 0., nothing to do
; fcut       : low cut-off frequency (no cal under this limit)
; fmin, fmax : limits for a filtering in a fixed system (for step >4)
; step       : level of calibration, 1-7 (4=SSL)
                               
; *** Example of cowave.in file 

; ./tha_scm_L1.rff
; ./cal
; 2037 11 30  # day-month-year of period
; 21 26 00.0  # Hour-min-sec   of start period (real double sec)
;  15.        # Duration of the time period (min)
; 256         # Number of point of the window (256-65536)
; 1           # Classic despin before detrending
; 0.          # detrend frequency, 0.=nothing to do
; 0.1         # Frequency cut-off for calibration
; 0. 0.       # Min/Max frequencies for filtering (0. 0. => full band)
; 4           # Processing levels asked (1-4)

; *** this file can be created by the following command:
;     (without the comments)

; scm_define_calparam, './tha_scm_L1.rff', './cal', Julsec1,Julsec2, $
;                         256,1, 0., 0.1, 0., 0., 4

; *** compute day-month-year of period and Hour-min-sec of start period 

  isodatim= tu_julsec_to_isodate(Julsec1, 8)
  tu_decode_isodate, isodatim,year,month,day,hour,min,secdbl
  duration= (Julsec2 -Julsec1)/60.D0

; *** create cowave.in

  GET_LUN, unit

  PRINT, ' '
  PRINT, FORMAT='(a,i4,a)', $
         'Creating cowave.in on unit',unit,', Please wait...'
  RFF_path= Dir_Results +RFF_name
  OPENW, unit, Dir_Results +'cowave.in'

  PRINTF, unit, format='(a)'                    , RFF_path
  PRINTF, unit, format='(a)'                    , Cal_path
  PRINTF, unit, format='(i4,2i3.2,T20,a)'       , year,month,day, $
                '# Year month day'
  PRINTF, unit, format='(i2.2,i3.2,f10.6,T20,a)', hour,min,secdbl, $
                '# Hour min sec (sec in DBLE)'
  PRINTF, unit, format='(f10.5,T20,a)'          , duration, $
                '# Duration in mn'
  PRINTF, unit, format='(i6,T20,a)'             , nbp, $
                '# Number of point of a calibration window'
  PRINTF, unit, format='(i2,T20,a)'             , despin, $
                '# 1=Classic despin before detrending, 0 => no despin'
  PRINTF, unit, format='(f9.4,T20,a)'           , fdet, $
                '# Detrend frequency (Hz), 0. => no detrend'
  PRINTF, unit, format='(f9.4,T20,a)'           , fcut, $
                '# Low cut frequency (Hz)'
  PRINTF, unit, format='(2f9.4,T20,a)'          , fmin,fmax, $
                '# Fmin, Fmax for filtering in fixed system 0.,0. => no'
  PRINTF, unit, format='(i2,T20,a)'             , step, $
                '# Calibration step (1-8, see below)'

  PRINTF, unit, ' '
  PRINTF, unit, '# Reminder on calibration step:'
  PRINTF, unit, '# ----------------------------'
  PRINTF, unit, '# 1: Volts,  spinning sensor system, with    DC field'
  PRINTF, unit, '# 2: Volts,  spinning sensor system, without DC field'
  PRINTF, unit, '# 3: nTesla, spinning sensor system, without DC field'
  PRINTF, unit, '# 4: nTesla, spinning SSL    system, without DC field'
  PRINTF, unit, '# 5: nTesla, fixed DSL system, without DC field'
  PRINTF, unit, '# 6: nTesla, fixed DSL system, with xy DC field'
  PRINTF, unit, '# 7: nTesla, fixed GSE system, without DC field, filtered <Fc'
  PRINTF, unit, '# 8: nTesla, fixed GSM system, without DC field, filtered <Fc'

  FREE_LUN, unit

  PRINT, FORMAT='(a)','Done...'
  PRINT

  END

; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
