; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

  PRO scm_get_sample_rate, satname, mode, time_mode_hed, $
                           val_sfreq, apid_str

; ----------------------------------------------------------------------
; Extract arrays of sample frequency and modes from SCM header file
; time resolution 512/sfreq
;
; O. Le Contel & P. Robert, CETP, February 2007
;
; input examples : satname='a'
;                  mode= 'scf'
; output example : time_mode_hed => time array associated
;                                   with the sample rate values
;                  val_sfreq => arrays of sample rate in Hz
;                  apid_str  > ='44c' for instance
; ----------------------------------------------------------------------

  PRINT, '------------------------------------------------'
  PRINT, 'Begin of scm_get_sample_rate'
  PRINT, 'Get sample rate and apid from SCM header file...'
  PRINT

  thm_load_scm, probe=satname, /get_support_data
  tplot_names

; compute argument of get_data

  thscs_mode_hed = 'th'+satname+'_'+mode+'_hed'

; extract apid
; Warning: APID is written in decimal (ex. 1092 dec --> 444 hex)

  get_data,thscs_mode_hed,time_mode_hed,val_mode_hed

  val_apid = $
             reform(uint((32b*val_mode_hed(*,0)/32b))*uint(256) $
           + uint(val_mode_hed(*,1)))
  ;;dec2hex
  apid_str = strtrim(string( val_apid(0,0), format='(Z)'), 2)
  PRINT, 'apid=',apid_str

; extract sample rate
  val_sfreq = 2.^(reform(val_mode_hed(*,14)/16b)+1)

  PRINT, 'Number of points in SCM header file:', N_elements(time_mode_hed)
  PRINT, 'First 5 values of sample frequency:', val_sfreq(0:4)
  PRINT, 'Values of apid : ', apid_str
  PRINT, 'Values of mode : ', mode


; tests on  modes

  IF(mode EQ 'scf' AND apid_str NE '444') THEN $
    PRINT, '*** scm_cal_sample_rate: mode EQ scf and apid NE 444 !!!'

  IF(mode EQ 'scp' AND apid_str NE '448') THEN $
    PRINT, '*** scm_cal_sample_rate: mode EQ scp and apid NE 448 !!!'

  IF(mode EQ 'scw' AND apid_str NE '44c') THEN $
    PRINT, '*** scm_cal_sample_rate: mode EQ scw and apid NE 44c !!!'

; test on sample rate

  srmin=MIN(val_sfreq, MAX=srmax)

  IF(apid_str EQ '444' AND (srmin LT 2 OR srmax GT 256)) THEN $
    PRINT,'*** scm_cal_sample_rate: apid=444 and Srate < 2 or > 256'

  IF(apid_str EQ '448' AND (srmin LT 2 OR srmax GT 256)) THEN $
    PRINT,'*** scm_cal_sample_rate: apid=448 and Srate < 2 or > 256'

  IF(apid_str EQ '44c' AND (srmin LT 512 OR srmax GT 8192)) THEN $
    PRINT,'*** scm_cal_sample_rate: apid=44c and Srate < 512 or > 8192'

  PRINT, 'End of scm_get_sample_rate'
  PRINT, '--------------------------'

  END

; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
