;+
;	Procedure:
;		thm_comp_scm_response
;
;	Purpose:
;		Compute the voltage gain (magnitude only) as a function of frequency
;	for a given THEMIS SCM sensor.
;
;	Calling Sequence:
;	thm_comp_scm_response, sensor, ff, resp

;	Arguements:
;		sensor	STRING, ignored.
;		ff	FLOAT[ N], array of frequencies at which to compute the sensor response.
;		resp	float[ N], array of |voltage gain| vs. frequency.
;
;	Notes:
;	-- none.
;
; $LastChangedBy: kenb-mac $
; $LastChangedDate: 2007-07-03 09:10:02 -0700 (Tue, 03 Jul 2007) $
; $LastChangedRevision: 987 $
; $URL $
;-
pro thm_comp_scm_response, sensor, ff, resp

; model THM SCM sensor response.
; based on FM1, X-Sensor.
	f1 = 40.	; single-pole high-pass corner frequency, Hz.
	f2 = 1650.	; single-pole, low-pass corner frequency, Hz.

	ff1 = ff/f1
	ff2 = ff/f2

	resp = sqrt( ff1*ff1/((1.+ff1*ff1)*(1. + ff2*ff2)))

return
end
