

pro thm_load_mom_cal_array,time,momraw,scpotraw,qf,shft,probe=probe,caldata=caldata,verbose=verbose,raw=raw

; Use single cal file for now.
thx = 'tha'
calsource = !themis
cal_relpathname = thx+'/l1/mom/0000/'+thx+'_l1_mom_cal_v01.sav'
cal_file = file_retrieve(cal_relpathname, _extra=calsource)
if file_test(cal_file) then  restore,file=cal_file,verbose=verbose,description=description  $
else message,'No moment cal file: ' + cal_file

dprint,dlevel=2,verbose=!themis.verbose,description, ' : ',cal_file
dprint,dlevel=4,verbose=!themis.verbose,caldata,phelp=3


if keyword_set(raw) then begin
   sfx='_raw'
   caldata.mom_scale = 1.
   caldata.scpot_scale = 1.
endif else sfx=''

one = replicate(1.d,n_elements(time))
bad = where((qf and 'c'x) ne 0,nbad)
if nbad gt 0 then one[bad] = !values.f_nan

    nt = n_elements(time)
    thx = 'th'+probe
    instrs= 'p'+['ei','ee','si','se'] +'m'
    sh = shft
    me = .511e6/299792d^2
    mi = 1836*me
    mass=[mi,me,mi,me]
    s2_cluge = [3,0,3,0]

    for i=0,3 do begin
       s2 = sh and 7
       sh = sh / 16
       instr = instrs[i]
       s2=s2_cluge[i]
       dprint,instrs[i],s2[0]

       dens  = momraw[*,0,i]  * caldata.mom_scale[0,i] * one * 2.^s2
       flux = fltarr(nt,3)
       for m = 0,2 do flux[*,m]  =  momraw[*,1+m,i]  * caldata.mom_scale[1+m,i] * one * 2.^s2 ;* 1e5
       mflux = fltarr(nt,6)
       for m = 0,5 do mflux[*,m]  =  momraw[*,4+m,i]  * caldata.mom_scale[4+m,i] *one * 2.^s2    ; * 1e2; * mass[i]
       eflux = fltarr(nt,3)
       for m = 0,2 do eflux[*,m]  =  momraw[*,10+m,i]  * caldata.mom_scale[10+m,i] * one * 2.^s2  ; * 1e1 ; * mass[i]

       store_data,thx+'_'+instr+'_density'+sfx,data={x:time,  y:dens}
       store_data,thx+'_'+instr+'_flux'+sfx, data={x:time,  y:flux }
       store_data,thx+'_'+instr+'_mftens'+sfx,  data={x:time,  y:mflux  }
       store_data,thx+'_'+instr+'_eflux'+sfx,  data={x:time,  y:eflux  }

    endfor

    scpot = scpotraw * caldata.scpot_scale
    store_data,thx+'_pxxm_pot'+sfx,data={x:time,  y:scpot }
    store_data,thx+'_pxxm_qf',data={x:time, y:qf}, dlimit={tplot_routine:'bitplot'}
end







;+
; Themis moment calibration routine.
; Author: Davin Larson
;-
pro thm_load_mom_cal,probes=probes,scs=scs, create=create, verbose=verbose

if not keyword_set(probes) then probes = ['a','b','c','d','e']

for s=0,n_elements(probes)-1 do begin
  thx = 'th'+probes(s)
  get_data,thx+'_mom_raw',ptr=p
  get_data,thx+'_mom_pot_raw',ptr=pot
  get_data,thx+'_mom_qf_raw',ptr

  if keyword_set(p) then begin
    thm_load_mom_cal_array,*p.x,*p.y,0 ,probe=probe

    dprint,dlevel=4,'Finished with cal on '+thx
  endif


endfor


end




;+
;Procedure: THM_LOAD_MOM
;
;Purpose:  Loads THEMIS moments data
;
;keywords:
;  probe = Probe name. The default is 'all', i.e., load all available probes.
;          This can be an array of strings, e.g., ['a', 'b'] or a
;          single string delimited by spaces, e.g., 'a b'
;  datatype = The type of data to be loaded, for this case, there is only
;          one option, the default value of 'fgm', so this is a
;          placeholder should there be more that one data type. 'all'
;          can be passed in also, to get all variables.
;  TRANGE= (Optional) Time range of interest  (2 element array), if
;          this is not set, the default is to prompt the user. Note
;          that if the input time range is not a full day, a full
;          day's data is loaded
;  level = the level of the data, the default is 'l1', or level-1
;          data. A string (e.g., 'l2') or an integer can be used. 'all'
;          can be passed in also, to get all levels.
;  CDF_DATA: named variable in which to return cdf data structure: only works
;          for a single spacecraft and datafile name.
;  VARNAMES: names of variables to load from cdf: default is all.
;  /GET_SUPPORT_DATA: load support_data variables as well as data variables
;                      into tplot variables.
;  /DOWNLOADONLY: download file but don't read it.
;  /valid_names, if set, then this routine will return the valid probe, datatype
;          and/or level options in named variables supplied as
;          arguments to the corresponding keywords.
;  files   named variable for output of pathnames of local files.
;  /VERBOSE  set to output some useful info
;Example:
;   thg_load_mom,/get_suppport_data,probe=['a', 'b']
;Notes:
;  Written by Davin Larson Jan 2007.
;  Updated keywords KRB Feb 2007
;
; $LastChangedBy$
; $LastChangedDate$
; $LastChangedRevision$
; $URL$
;-
pro thm_load_mom, probes=probe, datatype=datatype, trange=trange, all=all, $
                 level=level, verbose=verbose, downloadonly=downloadonly, $
                 varnames=varnames, valid_names = valid_names, raw=raw, $
                 progobj=progobj

thm_init

vb = keyword_set(verbose) ? verbose : 0
vb = vb > !themis.verbose
if vb ge 5 then printdat,!themis

vprobes = ['a','b','c','d','e']
vlevels = ['l1','l2']
vdatatypes=['mom']

if keyword_set(valid_names) then begin
    probe = vprobes
    level = vlevels
    datatype = vdatatypes
    return
endif

if not keyword_set(probe) then probe='*'
probe = strfilter(vprobes, probe ,delimiter=' ',/string)

if not keyword_set(datatype) then datatype='*'
datatype = strfilter(vdatatypes, datatype ,delimiter=' ',/string)

if not keyword_set(level) then level='*'
level = strfilter(vdatatypes, level ,delimiter=' ',/string)

addmaster=0

for s=0,n_elements(probe)-1 do begin
     thx = 'th'+ probe[s]

     pathformat = thx+'/l1/mom/YYYY/'+thx+'_l1_mom_YYYYMMDD_v01.cdf'
     if vb ge 4 then print,ptrace(),'pathformat: ',pathformat

     relpathnames = file_dailynames(file_format=pathformat,trange=trange,addmaster=addmaster)
     files = file_retrieve(relpathnames, _extra=!themis, progobj = progobj)

     if keyword_set(downloadonly) then continue

     suf=''
     suf='_raw'
     if 1 then begin
     cdfi = cdf_load_vars(files,varnames=varnames2,verbose=vb,/all);,/no_attributes)
     if not keyword_set(cdfi) then continue
     vns = cdfi.vars.name
     time = cdfi.vars[where(vns eq thx+'_mom_time')].dataptr
     mom  = cdfi.vars[where(vns eq thx+'_mom')].dataptr
     qf   = cdfi.vars[where(vns eq thx+'_mom_qf')].dataptr
     pot  = cdfi.vars[where(vns eq thx+'_mom_pot')].dataptr
     hed      = cdfi.vars[where(vns eq thx+'_mom_hed')].dataptr
     hed_time = cdfi.vars[where(vns eq thx+'_mom_hed_time')].dataptr
     shft_index = round(interp(findgen(n_elements(*hed_time)) ,*hed_time, *time))
     shft = (*hed)[*,14 ]+  256u*  (*hed)[*,15]
     shft = shft[shft_index]

     thm_load_mom_cal_array,*time,*mom,*pot,*qf,shft   ,probe = probe[s],raw=raw

     tplot_ptrs = ptr_extract(tnames(/dataquant))
     unused_ptrs = ptr_extract(cdfi,except=tplot_ptrs)
     ptr_free,unused_ptrs


     endif else begin

     cdf2tplot,file=files,all=all,suffix=suf,verbose=vb ,get_support_data=1 ;get_support_data    ; load data into tplot variables

     get_data,thx+'_mom'+suf,ptr=p
     get_data,thx+'_mom_pot'+suf,ptr=p_pot

     options,thx+'_mom_qf'+suf,tplot_routine='bitplot'

     get_data,thx+'_mom_hed'+suf,ptr=phed
     if keyword_set(phed) then begin
        store_data,thx+'_mom_CompCfg',data={x:phed.x, y:(*phed.y)[*,12] }, dlim={tplot_routine:'bitplot'}
        store_data,thx+'_mom_covers',data={x:phed.x, y:(*phed.y)[*,13] }, dlim={tplot_routine:'bitplot'}
        shft = (*phed.y)[*,14] + (*phed.y)[*,15]*256u
        store_data,thx+'_mom_shift',data={x:phed.x, y:shft }, dlim={tplot_routine:'bitplot',colors:'r'}
     endif
     thm_load_mom_cal,probe=probe[s]

     endelse

endfor

end
